@@ -1,5 +1,5 @@
 /* This is -*- C -*- */
-/* $Id: guppi-canvas-group.c,v 1.8 2001/01/16 21:13:05 trow Exp $ */
+/* $Id: guppi-canvas-group.c,v 1.15 2001/11/20 03:40:57 trow Exp $ */
 
 /*
  * guppi-canvas-group.c
@@ -27,10 +27,12 @@
  */
 
 #include <config.h>
-#include <guppi-memory.h>
-#include "guppi-group-view.h"
 #include "guppi-canvas-group.h"
 
+#include <guppi-memory.h>
+#include <guppi-convenient.h>
+#include "guppi-group-view.h"
+
 typedef struct _GuppiCanvasGroupPrivate GuppiCanvasGroupPrivate;
 struct _GuppiCanvasGroupPrivate {
 
@@ -41,39 +43,8 @@ struct _GuppiCanvasGroupPrivate {
 
 static GtkObjectClass *parent_class = NULL;
 
-enum {
-  ARG_0
-};
-
 static void
-guppi_canvas_group_get_arg (GtkObject * obj, GtkArg * arg, guint arg_id)
-{
-  switch (arg_id) {
-
-  default:
-    break;
-  };
-}
-
-static void
-guppi_canvas_group_set_arg (GtkObject * obj, GtkArg * arg, guint arg_id)
-{
-  switch (arg_id) {
-
-  default:
-    break;
-  };
-}
-
-static void
-guppi_canvas_group_destroy (GtkObject * obj)
-{
-  if (parent_class->destroy)
-    parent_class->destroy (obj);
-}
-
-static void
-guppi_canvas_group_finalize (GtkObject * obj)
+guppi_canvas_group_finalize (GtkObject *obj)
 {
   GuppiCanvasGroup *grp = GUPPI_CANVAS_GROUP (obj);
 
@@ -84,64 +55,34 @@ guppi_canvas_group_finalize (GtkObject * obj)
     parent_class->finalize (obj);
 }
 
-/*****************************************************************************/
+/***************************************************************************/
 
 static void
-position_subitem (GuppiCanvasGroup * cgrp, GuppiCanvasItem * item)
+position_subitem (GuppiCanvasItem *grp, GuppiCanvasItem *subitem)
 {
-  double scale;
-  gboolean posd;
   double x0, y0, x1, y1;
   gint cx0, cy0, cx1, cy1;
 
-  scale = guppi_canvas_item_scale (GUPPI_CANVAS_ITEM (cgrp));
-  posd = guppi_canvas_item_positioned (GUPPI_CANVAS_ITEM (cgrp));
+  if (guppi_canvas_item_positioned (grp) && guppi_canvas_item_positioned (subitem)) {
 
-  if (!(posd && guppi_canvas_item_positioned (item))) {
+    gnome_canvas_item_show (GNOME_CANVAS_ITEM (subitem));
+    guppi_canvas_item_get_bbox_pt (subitem, &x0, &y0, &x1, &y1);
 
-    gnome_canvas_item_hide (GNOME_CANVAS_ITEM (item));
+    guppi_canvas_item_pt2c (grp, x0, y0, &cx0, &cy0);
+    guppi_canvas_item_pt2c (grp, x1, y1, &cx1, &cy1);
 
+    guppi_2sort_i (&cx0, &cx1);
+    guppi_2sort_i (&cy0, &cy1);
+
+    guppi_canvas_item_set_bbox_c (subitem, cx0, cy0, cx1+1, cy1+1);
+    
   } else {
-
-    gnome_canvas_item_show (GNOME_CANVAS_ITEM (item));
-
-    guppi_canvas_item_get_bbox_pt (item, &x0, &y0, &x1, &y1);
-
-    guppi_canvas_item_pt2c (GUPPI_CANVAS_ITEM (cgrp), x0, y0, &cx0, &cy0);
-    guppi_canvas_item_pt2c (GUPPI_CANVAS_ITEM (cgrp), x1, y1, &cx1, &cy1);
-
-    guppi_canvas_item_set_bbox_c (item, cx0, cy0, cx1+1, cy1+1);
-    guppi_canvas_item_set_scale (item, scale);
-
+    gnome_canvas_item_hide (GNOME_CANVAS_ITEM (subitem));
   }
 }
 
 static void
-position_subitem_wrapper (GuppiCanvasItem * item, gpointer ptr)
-{
-  position_subitem (GUPPI_CANVAS_GROUP (ptr), item);
-}
-
-static void
-position_all_subitems (GuppiCanvasGroup * cgrp)
-{
-  GList *iter = GNOME_CANVAS_GROUP (cgrp)->item_list;
-
-  while (iter != NULL) {
-    GuppiCanvasItem *item = GUPPI_CANVAS_ITEM (iter->data);
-
-    /* We do this here just to make sure that it gets set.
-       It will in fact get set repeatedly, but that doesn't matter. */
-    guppi_canvas_item_set_position_handler (item,
-					    position_subitem_wrapper, cgrp);
-
-    position_subitem (cgrp, item);
-    iter = g_list_next (iter);
-  }
-}
-
-static void
-remove_if_gone_iter_fn (GuppiCanvasItem * item, gpointer ptr)
+remove_if_gone_iter_fn (GuppiCanvasItem *item, gpointer ptr)
 {
   GuppiGroupView *grp;
   GuppiElementView *view;
@@ -155,7 +96,7 @@ remove_if_gone_iter_fn (GuppiCanvasItem * item, gpointer ptr)
 }
 
 static void
-add_if_missing_iter_fn (GuppiElementView * view, gpointer ptr)
+add_if_missing_iter_fn (GuppiElementView *view, gpointer ptr)
 {
   GuppiCanvasGroup *canv_grp;
   GuppiCanvasItem *item;
@@ -171,45 +112,55 @@ add_if_missing_iter_fn (GuppiElementView * view, gpointer ptr)
 }
 
 static void
-add_remove_items (GuppiCanvasGroup * grp_item)
+add_remove_items (GuppiCanvasGroup *grp_item)
 {
   GuppiGroupView *grp;
 
-  grp =
-    GUPPI_GROUP_VIEW (guppi_canvas_item_view (GUPPI_CANVAS_ITEM (grp_item)));
+  grp = GUPPI_GROUP_VIEW (guppi_canvas_item_view (GUPPI_CANVAS_ITEM (grp_item)));
 
   /* First, remove superfluous items. */
-  guppi_canvas_group_foreach_item (grp_item, remove_if_gone_iter_fn,
-				   grp_item);
+  guppi_canvas_group_foreach_item (grp_item, remove_if_gone_iter_fn, grp_item);
 
   /* Then add in new items. */
   guppi_group_view_foreach (grp, add_if_missing_iter_fn, grp_item);
 }
 
-
 static void
-state_changed (GuppiCanvasItem * item)
+set_scale_cb (GuppiCanvasItem *item, gpointer closure)
 {
-  position_all_subitems (GUPPI_CANVAS_GROUP (item));
-  guppi_canvas_item_request_update (item);
+  double scale = *(double *)closure;
+  guppi_canvas_item_set_scale (item, scale);
 }
 
 static void
-view_changed (GuppiCanvasItem * item)
+changed_scale (GuppiCanvasItem *item, double scale)
+{
+  guppi_canvas_group_foreach_item (GUPPI_CANVAS_GROUP (item), set_scale_cb, &scale);
+
+  if (GUPPI_CANVAS_ITEM_CLASS (parent_class)->changed_scale)
+    GUPPI_CANVAS_ITEM_CLASS (parent_class)->changed_scale (item, scale);
+}
+
+static void
+changed_state (GuppiCanvasItem *item)
+{
+  //position_all_subitems (GUPPI_CANVAS_GROUP (item));
+
+  if (GUPPI_CANVAS_ITEM_CLASS (parent_class)->changed_state)
+    GUPPI_CANVAS_ITEM_CLASS (parent_class)->changed_state (item);
+}
+
+static void
+changed_view (GuppiCanvasItem *item)
 {
   add_remove_items (GUPPI_CANVAS_GROUP (item));
-  position_all_subitems (GUPPI_CANVAS_GROUP (item));
-  guppi_canvas_item_request_update (item);
+
+  if (GUPPI_CANVAS_ITEM_CLASS (parent_class)->changed_view)
+    GUPPI_CANVAS_ITEM_CLASS (parent_class)->changed_view (item);
 }
 
 static void
-set_scale (GuppiCanvasItem * item)
-{
-  position_all_subitems (GUPPI_CANVAS_GROUP (item));
-}
-
-static void
-foreach (GuppiCanvasItem * item, GuppiCanvasItemFunc fn, gpointer user_data)
+foreach (GuppiCanvasItem *item, GuppiCanvasItemFunc fn, gpointer user_data)
 {
   GList *iter;
 
@@ -225,8 +176,20 @@ foreach (GuppiCanvasItem * item, GuppiCanvasItemFunc fn, gpointer user_data)
   }
 }
 
+static void
+bbox_changed_hook (GuppiCanvasItem *item, gint cx0, gint cy0, gint cx1, gint cy1)
+{
+  GList *iter;
+
+  iter = GNOME_CANVAS_GROUP (item)->item_list;
+  while (iter != NULL) {
+    position_subitem (item, GUPPI_CANVAS_ITEM (iter->data));
+    iter = g_list_next (iter);
+  }
+}
+
 static gboolean
-double_click (GuppiCanvasItem * item,
+double_click (GuppiCanvasItem *item,
 	      guint button, guint state, double pt_x, double pt_y)
 {
   GList *iter;
@@ -248,7 +211,7 @@ struct foreach_toolkit {
 };
 
 static void
-foreach_toolkit_fn (GuppiCanvasItem * item, gpointer ptr)
+foreach_toolkit_fn (GuppiCanvasItem *item, gpointer ptr)
 {
   struct foreach_toolkit *foo = (struct foreach_toolkit *) ptr;
   guppi_canvas_item_foreach_class_toolkit (item, foo->fn, foo->user_data);
@@ -256,7 +219,7 @@ foreach_toolkit_fn (GuppiCanvasItem * item, gpointer ptr)
 
 
 static void
-foreach_class_toolkit (GuppiCanvasItem * item,
+foreach_class_toolkit (GuppiCanvasItem *item,
 		       void (*fn) (GuppiPlotToolkit *, gpointer),
 		       gpointer user_data)
 {
@@ -266,32 +229,64 @@ foreach_class_toolkit (GuppiCanvasItem * item,
 				   foreach_toolkit_fn, &foo);
 }
 
+static void
+changed_position_cb (GuppiElementView *view, gpointer closure)
+{
+  GuppiCanvasGroup *grp = GUPPI_CANVAS_GROUP (closure);
+  GuppiCanvasItem *item;
 
-/*****************************************************************************/
+  /* Yes, this is mildly inefficient since it requires us to do a linear search
+     every time we change position. */
+  item = guppi_canvas_group_find_by_view (grp, view);
+  position_subitem (GUPPI_CANVAS_ITEM (grp), item);
+}
 
 static void
-guppi_canvas_group_class_init (GuppiCanvasGroupClass * klass)
+add_hook (GuppiCanvasGroup *grp, GuppiCanvasItem *item)
+{
+  GuppiElementView *view = guppi_canvas_item_view (item);
+  
+  gtk_signal_connect (GTK_OBJECT (view),
+		      "changed_position",
+		      GTK_SIGNAL_FUNC (changed_position_cb),
+		      grp);
+}
+
+static void
+remove_hook (GuppiCanvasGroup *grp, GuppiCanvasItem *item)
+{
+  GuppiElementView *view = guppi_canvas_item_view (item);
+  gtk_signal_disconnect_by_func (GTK_OBJECT (view), GTK_SIGNAL_FUNC (changed_position_cb), grp);
+}
+
+
+/***************************************************************************/
+
+static void
+guppi_canvas_group_class_init (GuppiCanvasGroupClass *klass)
 {
   GtkObjectClass *object_class = (GtkObjectClass *) klass;
   GuppiCanvasItemClass *item_class = GUPPI_CANVAS_ITEM_CLASS (klass);
 
   parent_class = gtk_type_class (GUPPI_TYPE_CANVAS_ITEM);
 
-  object_class->get_arg = guppi_canvas_group_get_arg;
-  object_class->set_arg = guppi_canvas_group_set_arg;
-  object_class->destroy = guppi_canvas_group_destroy;
   object_class->finalize = guppi_canvas_group_finalize;
 
-  item_class->state_changed = state_changed;
-  item_class->view_changed = view_changed;
-  item_class->set_scale = set_scale;
+  klass->add_hook = add_hook;
+  klass->remove_hook = remove_hook;
+
+  item_class->changed_scale = changed_scale;
+  item_class->changed_state = changed_state;
+  item_class->changed_view  = changed_view;
+
   item_class->foreach = foreach;
+  item_class->bbox_changed_hook = bbox_changed_hook;
   item_class->double_click = double_click;
   item_class->foreach_class_toolkit = foreach_class_toolkit;
 }
 
 static void
-guppi_canvas_group_init (GuppiCanvasGroup * obj)
+guppi_canvas_group_init (GuppiCanvasGroup *obj)
 {
   GuppiCanvasGroupPrivate *p;
 
@@ -317,11 +312,11 @@ GtkType guppi_canvas_group_get_type (void)
   return guppi_canvas_group_type;
 }
 
-/*****************************************************************************/
+/***************************************************************************/
 
 GuppiCanvasItem *
-guppi_canvas_group_find_by_view (GuppiCanvasGroup * cgrp,
-				 GuppiElementView * view)
+guppi_canvas_group_find_by_view (GuppiCanvasGroup *cgrp,
+				 GuppiElementView *view)
 {
   GList *iter;
 
@@ -338,8 +333,7 @@ guppi_canvas_group_find_by_view (GuppiCanvasGroup * cgrp,
 
     /* Descend into any "subgroups" */
     if (GUPPI_IS_CANVAS_GROUP (item)) {
-      item =
-	guppi_canvas_group_find_by_view (GUPPI_CANVAS_GROUP (item), view);
+      item = guppi_canvas_group_find_by_view (GUPPI_CANVAS_GROUP (item), view);
       if (item)
 	return item;
     }
@@ -351,7 +345,7 @@ guppi_canvas_group_find_by_view (GuppiCanvasGroup * cgrp,
 }
 
 void
-guppi_canvas_group_foreach_item (GuppiCanvasGroup * cgrp,
+guppi_canvas_group_foreach_item (GuppiCanvasGroup *cgrp,
 				 GuppiCanvasItemFunc fn, gpointer user_data)
 {
   GList *iter;
