@@ -1,6 +1,6 @@
 /* This is -*- C -*- */
 /* vim: set sw=2: */
-/* $Id: guppi-attribute-flavor.c,v 1.9 2001/10/02 02:10:19 trow Exp $ */
+/* $Id: guppi-attribute-flavor.c,v 1.10 2001/11/16 16:59:06 trow Exp $ */
 
 /*
  * guppi-attribute-flavor.c
@@ -32,7 +32,10 @@
 #include <stdlib.h>
 #include "guppi-attribute-flavor.h"
 
+#include <gtk/gtk.h>
 #include "guppi-attribute-flavor-private.h"
+#include "guppi-attribute-widget.h"
+#include "guppi-metric-entry.h"
 #include "guppi-defaults.h"
 #include "guppi-memory.h"
 #include "guppi-rgb.h"
@@ -183,6 +186,21 @@ guppi_attribute_flavor_register_object_semantics (const gchar *name,
 
 /* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */
 
+void
+guppi_attribute_flavor_add_widget (GuppiAttributeFlavor flavor,
+				   AttributeWidgetFn widget_fn)
+{
+  GuppiAttributeFlavorInfo *info = guppi_attribute_flavor_get_info (flavor);
+
+  g_assert (info != NULL);
+  
+  g_assert (widget_fn != NULL);
+  g_assert (info->widget == NULL);
+
+  info->widget = widget_fn;
+}
+
+/* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */
 
 void
 guppi_attribute_flavor_add_string_serialization (GuppiAttributeFlavor flavor,
@@ -262,6 +280,23 @@ guppi_attribute_flavor_get_name (GuppiAttributeFlavor flavor)
     return "[UnknownFlavor]";
 }
 
+GtkWidget *
+guppi_attribute_flavor_get_widget (GuppiAttributeFlavor flavor, const gchar *key)
+{
+  if (0 <= flavor && flavor < flavor_count && flavor_info_buffer[flavor]->widget)
+    return flavor_info_buffer[flavor]->widget (key);
+  else
+    return gtk_label_new ("N/A");
+}
+
+GtkWidget *
+guppi_attribute_flavor_glade_custom_fn (const gchar *label, const gchar *type, const gchar *key)
+{
+  GuppiAttributeFlavor flavor = guppi_attribute_flavor_get_from_name (type);
+  g_return_val_if_fail (flavor != (GuppiAttributeFlavor)-1, NULL);
+  return guppi_attribute_flavor_get_widget (flavor, key);
+}
+
 GuppiAttributeFlavorInfo *
 guppi_attribute_flavor_get_info (GuppiAttributeFlavor flavor)
 {
@@ -324,7 +359,43 @@ boolean_str_exp (gpointer ptr)
   return guppi_strdup (((gboolean) GPOINTER_TO_INT (ptr)) ? "true" : "false");
 }
 
+static void
+boolean_bag_to_widget_cb (GuppiAttributeWidget *gaw, gpointer closure)
+{
+  GtkToggleButton *tb = GTK_TOGGLE_BUTTON (closure);
+  gboolean val;
+  guppi_attribute_widget_bag_get (gaw, NULL, &val);
+  gtk_toggle_button_set_active (tb, val);
+}
 
+static void
+boolean_toggled_cb (GtkToggleButton *tb, gpointer closure)
+{
+  GuppiAttributeWidget *gaw = GUPPI_ATTRIBUTE_WIDGET (closure);
+  gboolean val = gtk_toggle_button_get_active (tb);
+  guppi_attribute_widget_bag_set (gaw, NULL, val);
+}
+
+static GtkWidget *
+boolean_widget (const gchar *key)
+{
+  GtkWidget *gaw = guppi_attribute_widget_new (guppi_attribute_flavor_boolean (), key);
+  GtkWidget *w;
+
+  w = gtk_check_button_new ();
+  gtk_container_add (GTK_CONTAINER (gaw), w);
+  gtk_widget_show (w);
+  
+  gtk_signal_connect (GTK_OBJECT (gaw),
+		      "bag_to_widget",
+		      GTK_SIGNAL_FUNC (boolean_bag_to_widget_cb),
+		      w);
+  gtk_signal_connect (GTK_OBJECT (w),
+		      "toggled",
+		      GTK_SIGNAL_FUNC (boolean_toggled_cb),
+		      gaw);
+  return gaw;
+}
 
 GuppiAttributeFlavor
 guppi_attribute_flavor_boolean (void)
@@ -342,6 +413,8 @@ guppi_attribute_flavor_boolean (void)
 					      boolean_va2p,
 					      boolean_p2va);
 
+    guppi_attribute_flavor_add_widget (flavor, boolean_widget);
+
     guppi_attribute_flavor_add_string_serialization (flavor,
 						     boolean_str_exp,
 						     boolean_str_imp);
@@ -410,6 +483,54 @@ int_imp_str (const gchar *str, gpointer *ptr)
   return FALSE;
 }
 
+static void
+int_bag_to_widget_cb (GuppiAttributeWidget *gaw, gpointer closure)
+{
+  GtkEditable *w = GTK_EDITABLE (closure);
+  gchar *str;
+  gint val, pos=0;
+
+  guppi_attribute_widget_bag_get (gaw, NULL, &val);
+  str = guppi_strdup_printf ("%d", val);
+  gtk_editable_delete_text (w, 0, -1);
+  gtk_editable_insert_text (w, str, strlen (str), &pos);
+  guppi_free (str);
+}
+
+static void
+int_activate_cb (GtkEditable *w, gpointer closure)
+{
+  GuppiAttributeWidget *gaw = GUPPI_ATTRIBUTE_WIDGET (closure);
+  gchar *str;
+  gint val;
+  str = gtk_editable_get_chars (w, 0, -1);
+  val = atoi (str);
+  guppi_attribute_widget_bag_set (gaw, NULL, val);
+  g_free (str);
+}
+
+static GtkWidget *
+int_widget (const gchar *key)
+{
+  GtkWidget *gaw = guppi_attribute_widget_new (guppi_attribute_flavor_int (), key);
+  GtkWidget *w = gtk_entry_new ();
+  
+  gtk_container_add (GTK_CONTAINER (gaw), w);
+  gtk_widget_show (w);
+
+  gtk_signal_connect (GTK_OBJECT (gaw),
+		      "bag_to_widget",
+		      GTK_SIGNAL_FUNC (int_bag_to_widget_cb),
+		      w);
+
+  gtk_signal_connect (GTK_OBJECT (w),
+		      "activate",
+		      GTK_SIGNAL_FUNC (int_activate_cb),
+		      gaw);
+
+  return gaw;
+}
+
 GuppiAttributeFlavor
 guppi_attribute_flavor_int (void)
 {
@@ -426,6 +547,8 @@ guppi_attribute_flavor_int (void)
 					      int_va2p,
 					      int_p2va);
 
+    guppi_attribute_flavor_add_widget (flavor, int_widget);
+
     guppi_attribute_flavor_add_string_serialization (flavor,
 						     int_exp_str,
 						     int_imp_str);
@@ -492,6 +615,55 @@ uint32_imp_str (const gchar *str, gpointer *ptr)
   return FALSE;
 }
 
+static void
+uint32_bag_to_widget_cb (GuppiAttributeWidget *gaw, gpointer closure)
+{
+  GtkEditable *w = GTK_EDITABLE (closure);
+  gchar *str;
+  guint32 val;
+  gint pos=0;
+
+  guppi_attribute_widget_bag_get (gaw, NULL, &val);
+  str = guppi_strdup_printf ("%u", val);
+  gtk_editable_delete_text (w, 0, -1);
+  gtk_editable_insert_text (w, str, strlen (str), &pos);
+  guppi_free (str);
+}
+
+static void
+uint32_activate_cb (GtkEditable *w, gpointer closure)
+{
+  GuppiAttributeWidget *gaw = GUPPI_ATTRIBUTE_WIDGET (closure);
+  gchar *str;
+  guint32 val;
+  str = gtk_editable_get_chars (w, 0, -1);
+  val = (guint32) atoi (str);
+  guppi_attribute_widget_bag_set (gaw, NULL, val);
+  g_free (str);
+}
+
+static GtkWidget *
+uint32_widget (const gchar *key)
+{
+  GtkWidget *gaw = guppi_attribute_widget_new (guppi_attribute_flavor_uint32 (), key);
+  GtkWidget *w = gtk_entry_new ();
+  
+  gtk_container_add (GTK_CONTAINER (gaw), w);
+  gtk_widget_show (w);
+
+  gtk_signal_connect (GTK_OBJECT (gaw),
+		      "bag_to_widget",
+		      GTK_SIGNAL_FUNC (uint32_bag_to_widget_cb),
+		      w);
+
+  gtk_signal_connect (GTK_OBJECT (w),
+		      "activate",
+		      GTK_SIGNAL_FUNC (uint32_activate_cb),
+		      gaw);
+
+  return gaw;
+}
+
 GuppiAttributeFlavor
 guppi_attribute_flavor_uint32 (void)
 {
@@ -508,6 +680,8 @@ guppi_attribute_flavor_uint32 (void)
 					      uint32_va2p,
 					      uint32_p2va);
 
+    guppi_attribute_flavor_add_widget (flavor, uint32_widget);
+
     guppi_attribute_flavor_add_string_serialization (flavor,
 						     uint32_exp_str,
 						     uint32_imp_str);
@@ -599,6 +773,51 @@ string_imp_str (const gchar *str, gpointer *ptr)
   return TRUE;
 }
 
+static void
+string_bag_to_widget_cb (GuppiAttributeWidget *gaw, gpointer closure)
+{
+  GtkEditable *entry = GTK_EDITABLE (closure);
+  gchar *str;
+  gint pos = 0;
+  guppi_attribute_widget_bag_get (gaw, NULL, &str);
+  gtk_editable_delete_text (entry, 0, -1);
+  gtk_editable_insert_text (entry, str, strlen (str), &pos);
+  guppi_free (str);
+}
+
+static void
+string_activate_cb (GtkEditable *w, gpointer closure)
+{
+  GuppiAttributeWidget *gaw = GUPPI_ATTRIBUTE_WIDGET (closure);
+  gchar *str;
+  str = gtk_editable_get_chars (w, 0, -1);
+  guppi_attribute_widget_bag_set (gaw, NULL, str ? str : "");
+  g_free (str);
+}
+
+static GtkWidget *
+string_widget (const gchar *key)
+{
+  GtkWidget *gaw = guppi_attribute_widget_new (guppi_attribute_flavor_string (), key);
+  GtkWidget *w;
+
+  w = gtk_entry_new ();
+  gtk_container_add (GTK_CONTAINER (gaw), w);
+  gtk_widget_show (w);
+
+  gtk_signal_connect (GTK_OBJECT (gaw),
+		      "bag_to_widget",
+		      GTK_SIGNAL_FUNC (string_bag_to_widget_cb),
+		      w);
+
+  gtk_signal_connect (GTK_OBJECT (w),
+		      "activate",
+		      GTK_SIGNAL_FUNC (string_activate_cb),
+		      gaw);
+
+  return gaw;
+}
+
 GuppiAttributeFlavor
 guppi_attribute_flavor_string (void)
 {
@@ -615,6 +834,8 @@ guppi_attribute_flavor_string (void)
 					      string_va2p,
 					      string_p2va);
 
+    guppi_attribute_flavor_add_widget (flavor, string_widget);
+
     guppi_attribute_flavor_add_string_serialization (flavor,
 						     string_exp_str,
 						     string_imp_str);
@@ -728,6 +949,45 @@ guppi_attribute_flavor_double (void)
 
 /* ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** */
 
+static void
+dimension_bag_to_widget_cb (GuppiAttributeWidget *gaw, gpointer closure)
+{
+  double pt;
+  guppi_attribute_widget_bag_get (gaw, NULL, &pt);
+  guppi_metric_entry_set_pt_value (GUPPI_METRIC_ENTRY (closure), pt);
+}
+
+static void
+dimension_changed_cb (GuppiMetricEntry *w, gpointer closure)
+{
+  double pt = guppi_metric_entry_pt_value (w);
+  guppi_attribute_widget_bag_set (GUPPI_ATTRIBUTE_WIDGET (closure), NULL, pt);
+}
+
+static GtkWidget *
+dimension_widget (const gchar *key)
+{
+  GtkWidget *gaw;
+  GtkWidget *w;
+
+  gaw = guppi_attribute_widget_new (guppi_attribute_flavor_dimension (), key);
+  w = guppi_metric_entry_new ();
+  gtk_container_add (GTK_CONTAINER (gaw), w);
+  gtk_widget_show (w);
+
+  gtk_signal_connect (GTK_OBJECT (gaw),
+		      "bag_to_widget",
+		      GTK_SIGNAL_FUNC (dimension_bag_to_widget_cb),
+		      w);
+
+  gtk_signal_connect (GTK_OBJECT (w),
+		      "changed_value",
+		      GTK_SIGNAL_FUNC (dimension_changed_cb),
+		      gaw);
+
+  return gaw;
+}
+
 GuppiAttributeFlavor
 guppi_attribute_flavor_dimension (void)
 {
@@ -744,6 +1004,8 @@ guppi_attribute_flavor_dimension (void)
 					      double_va2p,
 					      double_p2va);
 
+    guppi_attribute_flavor_add_widget (flavor, dimension_widget);
+
     guppi_attribute_flavor_add_string_serialization (flavor,
 						     double_exp_str,
 						     double_imp_str);
@@ -886,6 +1148,57 @@ rgb_imp_str (const gchar *str, gpointer *ptr)
   return FALSE;
 }
 
+static void
+rgb_bag_to_widget_cb (GuppiAttributeWidget *gaw, gpointer closure)
+{
+  double c[4];
+  guint32 rgb;
+  gint r, g, b;
+
+  guppi_attribute_widget_bag_get (gaw, NULL, &rgb);
+  UINT_TO_RGB (rgb, &r, &g, &b);
+  c[0] = r / 255.0;
+  c[1] = g / 255.0;
+  c[2] = b / 255.0;
+  c[3] = 1.0;
+  
+  gtk_color_selection_set_color (GTK_COLOR_SELECTION (closure), c);
+}
+
+static void
+rgb_color_changed_cb (GtkColorSelection *w, gpointer closure)
+{
+  double c[4];
+  guint32 rgb;
+  gtk_color_selection_get_color (w, c);
+  rgb = RGB_TO_UINT ((gint)rint (c[0] * 255),
+		     (gint)rint (c[1] * 255),
+		     (gint)rint (c[2] * 255));
+  guppi_attribute_widget_bag_set (GUPPI_ATTRIBUTE_WIDGET (closure), NULL, c);
+}
+
+static GtkWidget *
+rgb_widget (const gchar *key)
+{
+  GtkWidget *gaw = guppi_attribute_widget_new (guppi_attribute_flavor_rgb (), key);
+  GtkWidget *w = gtk_color_selection_new ();
+
+  gtk_color_selection_set_opacity ((GtkColorSelection *) w, FALSE);
+  gtk_container_add (GTK_CONTAINER (gaw), w);
+  gtk_widget_show (w);
+
+  gtk_signal_connect (GTK_OBJECT (gaw),
+		      "bag_to_widget",
+		      GTK_SIGNAL_FUNC (rgb_bag_to_widget_cb),
+		      w);
+  gtk_signal_connect (GTK_OBJECT (w),
+		      "color_changed",
+		      GTK_SIGNAL_FUNC (rgb_color_changed_cb),
+		      gaw);
+
+  return gaw;
+}
+
 GuppiAttributeFlavor
 guppi_attribute_flavor_rgb (void)
 {
@@ -902,6 +1215,8 @@ guppi_attribute_flavor_rgb (void)
 					      uint32_va2p,
 					      uint32_p2va);
 
+    guppi_attribute_flavor_add_widget (flavor, rgb_widget);
+
     guppi_attribute_flavor_add_string_serialization (flavor,
 						     rgb_exp_str,
 						     rgb_imp_str);
@@ -933,6 +1248,49 @@ rgba_imp_str (const gchar *str, gpointer *ptr)
   return FALSE;
 }
 
+static void
+rgba_bag_to_widget_cb (GuppiAttributeWidget *gaw, gpointer closure)
+{
+  guint32 rgba;
+  gint r, g, b, a;
+
+  guppi_attribute_widget_bag_get (gaw, NULL, &rgba);
+  UINT_TO_RGBA (rgba, &r, &g, &b, &a);
+  gnome_color_picker_set_i8 (GNOME_COLOR_PICKER (closure), r, g, b, a);
+}
+
+static void
+rgba_color_set_cb (GnomeColorPicker *w, guint r16, guint g16, guint b16, guint a16, gpointer closure)
+{
+  guint32 rgba;
+  guint8 r, g, b, a;
+  gnome_color_picker_get_i8 (w, &r, &g, &b, &a);
+  rgba = RGBA_TO_UINT (r, g, b, a);
+  guppi_attribute_widget_bag_set (GUPPI_ATTRIBUTE_WIDGET (closure), NULL, rgba);
+}
+
+static GtkWidget *
+rgba_widget (const gchar *key)
+{
+  GtkWidget *gaw = guppi_attribute_widget_new (guppi_attribute_flavor_rgba (), key);
+  GtkWidget *w = gnome_color_picker_new ();
+
+  gnome_color_picker_set_use_alpha ((GnomeColorPicker *) w, TRUE);
+  gtk_container_add (GTK_CONTAINER (gaw), w);
+  gtk_widget_show (w);
+
+  gtk_signal_connect (GTK_OBJECT (gaw),
+		      "bag_to_widget",
+		      GTK_SIGNAL_FUNC (rgba_bag_to_widget_cb),
+		      w);
+  gtk_signal_connect (GTK_OBJECT (w),
+		      "color_set",
+		      GTK_SIGNAL_FUNC (rgba_color_set_cb),
+		      gaw);
+
+  return gaw;
+}
+
 GuppiAttributeFlavor
 guppi_attribute_flavor_rgba (void)
 {
@@ -949,6 +1307,8 @@ guppi_attribute_flavor_rgba (void)
 					      uint32_va2p,
 					      uint32_p2va);
 
+    guppi_attribute_flavor_add_widget (flavor, rgba_widget);
+
     guppi_attribute_flavor_add_string_serialization (flavor,
 						     rgba_exp_str,
 						     rgba_imp_str);
