@@ -1,4 +1,4 @@
-/* $Id: guppi-guile.h,v 1.12 2000/12/01 00:17:31 trow Exp $ */
+/* $Id: guppi-guile.h,v 1.14 2001/11/19 05:40:52 trow Exp $ */
 
 /* 
  * guppi-guile.h
@@ -27,7 +27,7 @@
 #ifndef _INC_GUPPI_GUILE_H
 #define _INC_GUPPI_GUILE_H
 
-#include <gtk/gtk.h>
+/* #include <gtk/gtk.h> */
 #include <stdlib.h>
 #include <libguile.h>
 #include <guile/gh.h>
@@ -35,7 +35,7 @@
 #include "guppi-enums.h"
 #include "guppi-fn-wrapper.h"
 
-#include <guppi-defs.h>
+#include  "guppi-defs.h"
 
 BEGIN_GUPPI_DECLS
 
@@ -129,4 +129,4 @@ END_GUPPI_DECLS
 
 #endif /* _INC_GUPPI_GUILE_H */
 
-/* $Id: guppi-guile.h,v 1.12 2000/12/01 00:17:31 trow Exp $ */
+/* $Id: guppi-guile.h,v 1.14 2001/11/19 05:40:52 trow Exp $ */
