@@ -1,4 +1,4 @@
-/* $Id: guppi-text-squeeze.h,v 1.5 2000/10/27 18:23:05 trow Exp $ */
+/* $Id: guppi-text-squeeze.h,v 1.8 2001/11/19 05:40:52 trow Exp $ */
 
 /*
  * guppi-text-squeeze.h
@@ -26,9 +26,8 @@
 #ifndef _INC_GUPPI_TEXT_SQUEEZE_H
 #define _INC_GUPPI_TEXT_SQUEEZE_H
 
-#include <gtk/gtk.h>
-
-#include <guppi-defs.h>
+#include <glib.h>
+#include  "guppi-defs.h"
 
 BEGIN_GUPPI_DECLS
 
@@ -40,4 +39,4 @@ END_GUPPI_DECLS
 
 #endif /* _INC_GUPPI_TEXT-SQUEEZE_H */
 
-/* $Id: guppi-text-squeeze.h,v 1.5 2000/10/27 18:23:05 trow Exp $ */
+/* $Id: guppi-text-squeeze.h,v 1.8 2001/11/19 05:40:52 trow Exp $ */
